<?php
/*
 * pfSense-update-repos.inc.php
 *
 * part of pfSense (https://www.pfsense.org)
 * Copyright (c) 2022 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require_once("/etc/inc/pkg-utils.inc");

$EWSFQDN = "https://ews.netgate.com/pfupdate";
$FQDN = "https://ews.netgate.com/v2/repo";
$refreshinterval = (24 * 3600); // 24 hours
$idfile = "/var/db/uniqueid";

/**
 * Return a value specified by a path of keys in a nested array, if it exists.
 * @param $arr array value to search
 * @param $path string path with '/' separators
 * @param $default mixed value to return if the path is not found
 * @returns mixed value at path or $default if the path does not exist or if the
 *          path keys an empty string and $default is non-null
 */
function local_array_get_path(array &$arr, string $path, $default = null) {
	$vpath = explode('/', $path);
	$el = $arr;
	foreach ($vpath as $key) {
		if (mb_strlen($key) == 0) {
			continue;
		}
		if (is_array($el) && array_key_exists($key, $el)) {
			$el = $el[$key];
		} else {
			return ($default);
		}
	}

	if (($default !== null) && ($el === '')) {
		return ($default);
	}

	return ($el);
}

/**
 * Return a value specified by path in the config, if it exists.
 * @param $path string path with '/' separators
 * @param $default mixed value to return if the path is not found
 * @returns mixed value at path or $default if the path does not exist or if the
 *          path keys an empty string and $default is non-null
 */
function local_config_get_path($config, string $path, $default = null) {
	return (local_array_get_path($config, $path, $default));
}

// Parse the received JSON data and save the repository information
function local_save_repos($json, $repo_path) {
	global $g;

	$rtrn = array( "error" => -1, "count" => 0 );
	if ($json == NULL || $repo_path == NULL) {
		return ($rtrn);
	}
	$repos = json_decode($json, true);
	if ($repos == NULL || !is_array($repos)) {
		return ($rtrn);
	}

	/* Check for the backend messages. */
	if (isset($repos['messages'])) {
		$rtrn['messages'] = $repos['messages'];
	}

	/* Empty list ? */
	if (!is_array($repos['repos']['repo'])) {
		$rtrn['error'] = 0;
		return ($rtrn);
	}

	remove_repos_dir("{$repo_path}-tmp");
	$basename = "{$repo_path}-tmp/{$g['product_name']}-repo";

	$repos_count = 0;
	foreach ($repos["repos"]["repo"] as $repo) {

		if (!isset($repo['abi']) || !isset($repo['altabi']) ||
		    !isset($repo['conf']) || !isset($repo['descr']) ||
		    !isset($repo['name'])) {
			continue;
		}

		$reponame = "{$basename}-{$repo['name']}";
		file_put_contents($reponame . ".abi", $repo['abi']);
		file_put_contents($reponame . ".altabi", $repo['altabi']);
		file_put_contents($reponame . ".conf", base64_decode($repo['conf']));
		file_put_contents($reponame . ".descr", $repo['descr']);
		file_put_contents($reponame . ".name", $repo['name']);
		if (isset($repo['help']) && !empty($repo['help'])) {
			file_put_contents($reponame . ".help", $repo['help']);
		}
		// Save fingerprint file
		if (isset($repo['fingerprint']) && !empty($repo['fingerprint'])) {
			if (isset($repo['fpname']) && !empty($repo['fpname'])) {
				$fppath = "/usr/local/share/pfSense/keys/pkg/trusted/{$repo['fpname']}";
			} else {
				$fppath = "/usr/local/share/pfSense/keys/pkg/trusted/{$repo['name']}.pfsense.org";
			}

			file_put_contents($fppath, $repo['fingerprint']);
		}
		// Save the client Cert & key
		if (isset($repo['cert']) && !empty($repo['cert'])) {
			write_x509_cert($reponame . "-cert.pem", $repo['cert']);
		}
		if (isset($repo['key'])  && !empty($repo['key'])) {
			write_x509_rsa_key($reponame . "-key.pem", $repo['key']);
		}
		if (isset($repo['default'])) {
			file_put_contents($reponame . ".default", "1");
		}

		$repos_count++;
	}

	if ($repos_count > 0) {
		if (file_exists("{$repo_path}-old")) {
			remove_repos_dir("{$repo_path}-old");
		}
		if (file_exists($repo_path)) {
			rename("{$repo_path}", "${repo_path}-old");
		}
		rename("{$repo_path}-tmp", "${repo_path}");
	}

	$rtrn['error'] = 0;
	$rtrn['count'] = $repos_count;
	return ($rtrn);
}

/*
 * Update the list of available repositories from the server. This will allow
 * migration to another update repository should the existing one become
 * unavailable
 */
function local_update_repos($debug = false) {
	global $FQDN, $g, $idfile;

	/*
	 * Gather some information about the system so the proper
	 * repo can be returned
	 */
	$nid = file_get_contents($idfile);
	$serial = system_get_serial();

	$filesystems = get_mounted_filesystems();
	$fstype = "";

	foreach ($filesystems as $fs)  {
		if ($fs['mountpoint'] != "/") {
			continue;
		}
		$fstype = strtoupper($fs['type']);
		break;
	}

	// Compose a version string in JSON format
	$os = php_uname('s');
	$osver = php_uname('r');
	$ed =  (strpos($g['product_label'], 'Plus') !== false)
	    ? "Plus" : "Community";
	$pkglist = get_pkg_info("all", false, true);
	$platform = system_identify_specific_platform();
	$platformname = gettext('Unknown system');

	if (isset($platform['descr'])) {
		$platformname = $platform['descr'];
	}

	$arch =  php_uname('m');
	$config = parse_config();
	$locale = local_config_get_path($config, 'system/language', 'en_US');

	$va = array();
	$va['platform'] = $platformname;
	$va['os'] = $os;
	$va['osver'] = $platformname;
	$va['fstype'] = $fstype;
	$va['prod'] = $g['product_label'];
	$va['ver'] = $g['product_version_string'];
	$va['ed'] = $ed;
	$va['pkgs'] = array();

	foreach($pkglist as $pkg) {
		$va['pkgs'][] = array('name' => $pkg['shortname'], 'ver' => $pkg['version']);
	}

	$post = [
	    'uid' => $nid,
	    'language' => $locale,
	    'serial' => $serial,
	    'version' => json_encode($va),
	    'arch' => $arch
	];

	$ch = curl_init();
	curl_setopt($ch, CURLOPT_HEADER, 0);
	curl_setopt($ch, CURLOPT_VERBOSE, 0);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_USERAGENT, $g['product_label'] . '/' . $g['product_version']);
	curl_setopt($ch, CURLOPT_URL, $FQDN);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post));
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT ,4);
	set_curlproxy($ch);

	$response = curl_exec($ch);
	$status = curl_getinfo($ch, CURLINFO_HTTP_CODE);

	curl_close($ch);

	if ($debug) {
		echo "status: $status\n";
		echo "response: $response\n";
	}
	if ($status != 200 || $response == false) {
		return (array( "error" => 1, "count" => 0,
		    "messages" => array("We could not connect to Netgate servers. Please try again later.")));
	}

	return (local_save_repos($response, $g['pkg_repos_path']));
}

?>
